import time
import os, sys
from watchdog.observers import Observer
import http.server
import socketserver
import webbrowser
import threading
import shutil

from ..util import colorify


def watch_directory(directory, event_handler):
    observer = Observer()
    observer.schedule(event_handler, directory, recursive=True)
    observer.start()
    print(f"{colorify.green("[   OK   ]")} watching '{directory}' for changes...")
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()


class MyHTTPServer(threading.Thread):
    def __init__(self, directory, port=8000):
        super().__init__()
        self.directory = directory
        self.port = port
        self.server = None

    def run(self):
        directory = self.directory
        class Handler(http.server.SimpleHTTPRequestHandler):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, directory=directory, **kwargs)

        with socketserver.TCPServer(("", self.port), Handler) as self.server:
            print(f"{colorify.yellow("[ SERVER ]")} Serving HTTP on 0.0.0.0 port {self.port} (http://0.0.0.0:{self.port}/) ...")
            webbrowser.open(f"http://localhost:{self.port}")
            try:
                self.server.serve_forever()
            except KeyboardInterrupt:
                print(f"\n{colorify.gray("[ SERVER ]")} Shutting down the server...")
                self.server.server_close()



def clean_directory(directory, exceptions=[]):
    for item in os.listdir(directory):
        if item not in exceptions:
            item_path = os.path.join(directory, item)
            if os.path.isdir(item_path):
                shutil.rmtree(item_path)
            else:
                os.remove(item_path)


def path_join(*args):
    return os.path.join(*args).replace("\\", "/")


def is_file_under_folder(path, filepath):
    paths = []
    for subdir, dirs, files in os.walk(path):
        for file in files:
            src_path = path_join(subdir, file)
            paths.append(src_path)
    
    if filepath in paths:
        return True
    
    return False



#!/usr/bin/env python3
# coding=utf-8

"""
Copyright (c) 2024 suyambu developers (http://suyambu.net/gasper)
See the file 'LICENSE' for copying permission
"""

from peewee import *


def generate(gasper, db, table, host, port, username, password, where=None, limit=None, order=None, only=None, unique=False, plugin=None):
    database = MySQLDatabase(db, host=host, port=port, user=username, passwd=password)
    cursor = database.execute_sql(f"SHOW COLUMNS FROM {db}.{table};")
    col_names = cursor.fetchall()
    output = []
    query = f"SELECT * FROM {table}"
    if where:
        query += f" WHERE {where}"
    if order:
        query += f" ORDER {order}"
    if limit:
        query += f" LIMIT {limit}"
    cursor = database.execute_sql(query)
    for row in cursor.fetchall():
        output.append({})
        for i, col in enumerate(row):
            output[-1][col_names[i][0]] = col
    
    if only is not None:
        tmp = []
        cols = list(map(lambda x: x.strip(), only.split(",")))
        for out in output:
            tmp.append({})
            for k, v in out.items():
                if k in cols:
                    tmp[-1][k] = v
        output = tmp
        
    if unique:
        tmp = []
        matches = []
        for out in output:
            m = "".join(map(lambda x: str(x), out.values()))
            if m in matches:
                continue
            matches.append(m)
            tmp.append({})
            for k, v in out.items():
                tmp[-1][k] = v
                    
        output = tmp

    if plugin is not None:
        m = __import__(plugin)
        output = m.plugin(output)

    return output



if __name__ == "__main__":
    generate(db="automator", table="user", host="localhost", port=3306, username="root", password="rockjeev")



#!/usr/bin/env python3
# coding=utf-8

"""
Copyright (c) 2024 suyambu developers (http://suyambu.net/gasper)
See the file 'LICENSE' for copying permission
"""

import os
from datetime import datetime
from markdown import markdown


def generate(gasper, path, plugin=None):
    output = []
    for subdir, dirs, files in os.walk(path):
        for file in files:
            src_path = os.path.join(subdir, file)
            filename = os.path.basename(src_path)
            nodes = filename.split("-")
            year = int(nodes.pop(0))
            month = int(nodes.pop(0))
            day = int(nodes.pop(0))
            
            filename = "-".join(nodes)
            date = datetime(year, month, day)
            formated_date = date.strftime("%b %d, %Y")
            
            nodes = filename.split(".")
            nodes.pop()
            filename = ".".join(nodes)
            
            matter = {
                "date": formated_date,
                "filename": filename,
                "permalink": f"{year}/{month}/{day}/{filename}"
            }
            dummy = matter
            for _ in range(2):
                (dummy, _) = gasper.extract_frontmatter(src_path, dummy, debug=True)
            (init_matter, content) = gasper.extract_frontmatter(src_path, dummy, debug=True)
            matter.update(init_matter)
            
            content = gasper.env.from_string(content).render(page=matter, content=None, site=gasper.config)
            
            if src_path.endswith(".md") or src_path.endswith(".markdown"):
                content = markdown(content, extensions=['fenced_code'])
            
            matter["content"] = content
            output.append(matter)
    
    if plugin is not None:
        m = __import__(plugin)
        output = m.plugin(output)
    
    return output


import os
from shutil import *


def copytree(src, dst, symlinks=False, ignore=None):
    names = os.listdir(src)
    if ignore is not None:
        ignored_names = ignore(src, names)
    else:
        ignored_names = set()

    if not os.path.isdir(dst): # This one line does the trick
        os.makedirs(dst)
    errors = []
    for name in names:
        if name in ignored_names:
            continue
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        try:
            if symlinks and os.path.islink(srcname):
                linkto = os.readlink(srcname)
                os.symlink(linkto, dstname)
            elif os.path.isdir(srcname):
                copytree(srcname, dstname, symlinks, ignore)
            else:
                # Will raise a SpecialFileError for unsupported file types
                copy2(srcname, dstname)
        # catch the Error from the recursive copytree so that we can
        # continue with other files
        except Error as err:
            errors.extend(err.args[0])
    try:
        copystat(src, dst)
    except OSError as why:
        if WindowsError is not None and isinstance(why, WindowsError):
            # Copying file access times may fail on Windows
            pass
        else:
            errors.extend((src, dst, str(why)))
    if errors:
        raise Error(errors)


#!/usr/bin/env python3
# coding=utf-8

"""
Copyright (c) 2024 suyambu developers (http://suyambu.net/gasper)
See the file 'LICENSE' for copying permission
"""

def format_percentage(current, total):
    per = str(round(current/total*100, 1))
    if len(per) == 3:
        return "  {}%".format(per)
    elif len(per) == 4:
        return " {}%".format(per)

    return per


def format_space(s, count):
    return s + (" " * (count - len(s)))


#!/usr/bin/env python3
# coding=utf-8

"""
Copyright (c) 2024 suyambu developers (http://suyambu.net/gasper)
See the file 'LICENSE' for copying permission
"""


from .colorify import blue, red, green, yellow, green_bg, black, bold


def info(s):
    print("[{}] {}".format(bold(blue("*")), s))


def warn(s):
    print("[{}] {}".format(bold(yellow("!")), s))


def error(s):
    print("[{}] {}".format(bold(red("-")), s))


def success(s):
    print("[{}] {}".format(bold(green("+")), s))


#!/usr/bin/env python3
# coding=utf-8

"""
Copyright (c) 2024 suyambu developers (http://suyambu.net/gasper)
See the file 'LICENSE' for copying permission
"""

import sys, os
from jinja2 import Environment, BaseLoader
from markdown import markdown
import pathlib
import re
import yaml
from slugify import slugify
import json
from watchdog.events import FileSystemEventHandler
import requests
import base64
import time
import copy
from bs4 import BeautifulSoup

from .libs.argparse import ArgParse
from .util import ext, colorify
from .generator.mysql import generate as mysql_generate
from .generator.posts import generate as posts_generate
from .generator.json_parser import generate as json_generate
from .core.loader import GasperLoader
from .core import io
from .core.copytree import copytree


sys.path.insert(0, "site/_plugins")


class Gasper(FileSystemEventHandler):
    def __init__(self):
        yaml.SafeLoader.add_constructor("tag:yaml.org,2002:python/unicode", self.yaml_constructor)
        self.global_matter = {}
        self.path = None
        self.args = self.parse_arguments()
        # self.env = Environment(loader=FileSystemLoader(self.path), autoescape = True)
        self.config = self.load_config()
        self.env = Environment(loader=GasperLoader(io.path_join(self.path, "_include"), self))
        self.apply_filters()
        self.allowed_extension = [
            "html",
            "txt",
            "xml",
            "json",
            "exe",
            "bin"
        ]
        self.sitemap = []
        self.load_sitemap()
    
    def load_sitemap(self):
        sitemap_path = io.path_join("dist", "sitemap.xml")
        if os.path.exists(sitemap_path):
            with open(sitemap_path, "r") as f:
                soup = BeautifulSoup(f.read(), features="xml")
            locs = set(soup.select("url loc"))
            
            for loc in locs:
                self.sitemap.append(loc.text)            

    def yaml_constructor(self, loader, node):
        return node.value
        
    def task_print(self, s):
        print(f"{colorify.gray("[  TASK  ]")} {s}")
    
    def error_print(self, s):
        print(f"{colorify.red("[ EXCEPT ]")} {s}")
        
    def success_print(self, s):
        print(f"{colorify.green("[   OK   ]")} {s}")
        
    def load_config(self):
        config_path = io.path_join(self.path, "_config.yaml")
        config = {}
        if os.path.exists(config_path):
            with open(config_path, "r") as f:
                config = yaml.safe_load(f.read())
        return config
    
    def apply_filters(self):
        self.env.filters["slugify"] = slugify
        self.env.globals["json"] = json
        self.env.filters["b64encode"] = self.b64encode
        self.env.filters["b64decode"] = base64.b64decode

    def b64encode(self, s):
        return base64.b64encode(s.encode()).decode()
        
    def get_file_path_from_src_path(self, src_path):
        file_path = src_path.replace(self.path, "")
        if file_path.startswith("\\") or file_path.startswith("/"):
            file_path = file_path[1:]
        file_path = file_path.replace("\\", "/")
        return file_path
    
    def generate_content(self, src_path, prev_content=None, prev_matter={}):
        file_path = self.get_file_path_from_src_path(src_path)
        
        # dummy, _ = self.extract_frontmatter(file_path, prev_matter)
        # dummy.update(prev_matter)
        matter, output = self.extract_frontmatter(file_path, prev_matter)
        
        if matter is None:
            matter = {}

        matter.update(prev_matter)
        try:
            self.global_matter = matter
            if "parser" in matter and matter.get("parser") == "text":
                content = self.env.from_string(output).render(page=matter, content=prev_content, site=self.config)
            else:
                content = markdown(self.env.from_string(output).render(page=matter, content=prev_content, site=self.config), extensions=['fenced_code'])
        except json.decoder.JSONDecodeError:
            print(f"Error: {matter["title"]}")
            return output
        
        if matter.get("trimContent"):
            content = content.strip()
        
        if "layout" in matter:
            if os.path.exists(io.path_join(self.path, "_layout", matter["layout"] + ".md")):
                matter["layout"] = io.path_join(self.path, "_layout", matter["layout"] + ".md")
            elif os.path.exists(io.path_join(self.path, "_layout", matter["layout"] + ".markdown")):
                matter["layout"] = io.path_join(self.path, "_layout", matter["layout"] + ".markdown")
            elif os.path.exists(io.path_join(self.path, "_layout", matter["layout"] + ".html")):
                matter["layout"] = io.path_join(self.path, "_layout", matter["layout"] + ".html")
            layout = matter.pop("layout")
            return self.generate_content(layout, content, matter)
        
        return content
    
    def handle_generator_related(self, generator, rows, row, index):
        related = {}
        if generator["from"] == "mysql":
            if generator.get("related"):
                s = yaml.dump(generator.get("related")).replace("\\u2192\\u2192", "{{").replace("\\u2190\\u2190", "}}")
                s = self.env.from_string(s).render(page={ 
                    "generator": {
                        "row": row,
                        "index": index,
                        "count": len(rows),
                        "related": {},
                        "rows": rows
                    } 
                }, content=None, site=self.config)
                tmp_related = yaml.safe_load(s)
                for r in tmp_related:
                    related_rows = mysql_generate(self, generator["db"], r["table"], generator["host"], generator["port"], generator["username"], generator["password"], r.get("where"), r.get("limit"), r.get("order"), r.get("only"), r.get("unique", False), plugin=r.get("plugin"))
                    related[r["table"]] = related_rows
        return related

    def dumpTo(self, generator, rows, related):
        if generator.get("dumpTo"):
            self.task_print(f"Dumping to {generator.get("dumpTo")}")
            dump_path = io.path_join("dist", generator["dumpTo"])
            if not os.path.exists(os.path.dirname(dump_path)):
                os.makedirs(os.path.dirname(dump_path))
            
            with open(dump_path, "wb+") as f:
                f.write(json.dumps({ "rows": rows, "related": related }).encode("utf-8"))
    
    def handle_generator(self, generator):
        if generator["from"] == "mysql":
            rows = mysql_generate(self, generator["db"], generator["table"], generator["host"], generator["port"], generator["username"], generator["password"], generator.get("where"), generator.get("limit"), generator.get("order"), generator.get("only"), generator.get("unique", False), plugin=generator.get("plugin"))

            return rows
        elif generator["from"] == "posts":
            path = generator.get("path")
            if path is None:
                path = "_posts"
            path = io.path_join(self.path, path)
            posts = posts_generate(self, path, plugin=generator.get("plugin"))
            return posts
        elif generator["from"] == "json":
            path = generator.get("path")
            path = io.path_join(self.path, "_data", path)
            return json_generate(self, path, exclude=generator.get("exclude"), limit=generator.get("limit"), plugin=generator.get("plugin"), reverse=generator.get("reverse"))
        
    def generate(self, src_path, dist_path, permalink=None, init_matter={}):
        html = self.generate_content(src_path, prev_content=None, prev_matter=init_matter)
        
        output_path = io.path_join(dist_path, "index.html")
        if permalink is not None:
            if permalink.startswith("/"):
                permalink = permalink[1:]
            if permalink.split(".")[-1] in self.allowed_extension:
                output_path = f"dist/{permalink}"
            else:
                output_path = f"dist/{permalink}/index.html"

        if not os.path.exists(os.path.dirname(output_path)):
            os.makedirs(os.path.dirname(output_path))
        with open(output_path, "wb") as f:
            f.write(html.encode("utf-8"))
            
        if "sitemap" in self.config and init_matter.get("sitemap") != "ignore":
            nodes = list(pathlib.Path(output_path).parts)
            nodes.pop(0)
            if nodes[-1] == "index.html":
                nodes.pop()
            final_url = self.config.get("url") + "/" + "/".join(nodes)
            if len(nodes) > 0 and "." not in nodes[-1]:
                final_url += "/"
            if final_url not in self.sitemap:
                self.sitemap.append(final_url)

            s = f"""<?xml version="1.0" encoding="UTF-8"?>
<urlset
    xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9
            http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd">
            """
            for m in self.sitemap:
                level = "1.0"
                parts = m.split(self.config.get("url"))
                parts = parts[-1][1:].split("/")
                if len(parts) <= 1:
                    level = "1.0"
                elif len(parts) <= 2:
                    level = "0.8"
                else:
                    level = "0.7"
                s += f"""<url>
  <loc>{m}</loc>
  <priority>{level}</priority>
</url>
"""
            s += "</urlset>"
            with open(io.path_join("dist", self.config.get("sitemap")), "wb+") as f:
                f.write(s.encode("utf-8"))
            
        self.task_print(f"{src_path} → {output_path}")

    def build(self):
        if not os.path.exists("dist"):
            os.mkdir("dist")
        
        only = None
        if self.args.only:
            only = io.path_join(self.path, self.args.only)
        else:
            io.clean_directory("dist", ["static", ".git", "CNAME"])
        
        self.task_print("moving static files...")
        if os.path.exists(io.path_join(self.path, "_static")):
            copytree(io.path_join(self.path, "_static"), io.path_join("dist", "static"))
        
        if not os.path.exists("dist"):
            os.mkdir("dist")
        for subdir, dirs, files in os.walk(self.path):
            for file in files:
                if file.startswith("_"):
                    continue
                src_path = io.path_join(subdir, file)

                if only is not None and os.path.isdir(only) and not io.is_file_under_folder(only, src_path):
                    continue
                elif only is not None and os.path.isfile(only) and src_path != only:
                    continue
                if ext.any_startswith("_", pathlib.Path(src_path).parts):
                    continue
                
                dist_path = io.path_join(subdir, file).replace(self.path, "dist")
                if dist_path.endswith("index.md") or dist_path.endswith("index.markdown") or dist_path.endswith("index.html"):
                    dist_path = os.path.dirname(dist_path)
                elif dist_path.endswith(".md") or dist_path.endswith(".html"):
                    file_name = pathlib.Path(dist_path).name
                    dist_path = io.path_join(os.path.dirname(dist_path), file_name.split(".md")[0].split(".html")[0].split(".markdown")[0])
                                
                file_path = self.get_file_path_from_src_path(src_path)
                dummy, _ = self.extract_frontmatter(file_path, is_raw=True)
                generator_copy = copy.deepcopy(dummy)
                rows = None
                related = {}
                if "generator" in dummy:
                    rows = self.handle_generator(dummy["generator"])

                if dummy.get("generator", None) is not None and dummy["generator"].get("skip") == True:
                    related = self.handle_generator_related(generator_copy["generator"], rows, None, -1)
                    self.dumpTo(generator_copy["generator"], rows, related)
                    dummy["generator"] = {
                        "row": None,
                        "index": -1,
                        "count": len(rows),
                        "related": related,
                        "rows": rows
                    }
                    init_matter, _ = self.extract_frontmatter(file_path, dummy)
                    init_matter["generator"] = dummy["generator"]
                    self.generate(src_path, dist_path, init_matter.get("permalink", None), init_matter=init_matter)
                elif rows is not None:
                    total_rows = len(rows)
                    for index, row in enumerate(rows):
                        # print(index, "\n\n")
                        # input()
                        related = self.handle_generator_related(generator_copy["generator"], rows, row, index)
                        dummy["generator"] = {
                            "row": row,
                            "index": index,
                            "count": total_rows,
                            "related": related,
                            "rows": rows
                        }
                        init_matter, _ = self.extract_frontmatter(file_path, dummy)
                        init_matter["generator"] = dummy["generator"]
                        self.generate(src_path, dist_path, init_matter.get("permalink", None), init_matter=init_matter)
                else:
                    init_matter, _ = self.extract_frontmatter(file_path, dummy)
                    self.generate(src_path, dist_path, init_matter.get("permalink", None), init_matter=init_matter)    

    def extract_frontmatter(self, path, prev_matter={}, is_raw=False, debug=False):
        if os.path.exists(path):
            abspath = path
        else:
            abspath = io.path_join(self.path, path)

        with open(abspath, "r", encoding="utf-8") as f:
            content = f.read()

        pattern = re.compile(r'---\n(.*?)\n---', re.DOTALL)
        m = re.search(pattern, content)
        matter = {}
        output = content
        if m:
            if is_raw:
                s = m.group(1).replace("{{", "→→").replace("}}", "←←")
            else:
                s = self.env.from_string(m.group(1)).render(page=prev_matter, content=None, site=self.config)
            matter = yaml.safe_load(s)
            output = re.sub(pattern, "", output)
        return (matter, output)
    
    def parse_arguments(self):
        parser = ArgParse(argument_space_count=20, usage="gasper [path] [options]")
        parser.add_argument(["--help", "-h"], description="show help", is_flag=True)
        parser.add_argument(["--watch", "-w"], description="build and watch for changes", is_flag=True)
        parser.add_argument(["--build", "-b"], description="build the site", is_flag=True)
        parser.add_argument(["--only"], description="only build under this directory")
        args = parser.parse()
        if len(sys.argv) < 1 or not os.path.exists(sys.argv[1]) or os.path.isfile(sys.argv[1]):
            parser.print_help()
            sys.exit(1)
        
        if args.help:
            parser.print_help()
            sys.exit(0)

        self.path = sys.argv[1]
        if self.path.endswith("\\") or self.path.endswith("/"):
            self.path = self.path[:-1]
        
        if not args.watch and not args.build:
            parser.print_help()
            sys.exit(1) 
        
        self.path = self.path.replace("\\", "/")
        if args.only:
            args.only = args.only.replace("\\", "/")
        return args
    
    def handle(self):
        # try:
        self.build()
        # except Exception as e:
        #     self.error_print(e)
            
    def exit(self):
        print(self.is_terminated)
        while not self.is_terminated:
            time.sleep(1)
            r = requests.get("http://localhost:8080")
            print(r.status_code)

    def run(self):
        if self.args.watch:
            self.handle()
            
            # http_server = io.MyHTTPServer("dist", 8080)
            # http_server.start()
            io.watch_directory(self.path, self)

            # try:
            #     inp = input("Do you really want quit? (Y/n) ").strip().lower()
            #     if inp == "" or inp == "y" or inp == "yes":
            #         raise Exception() 
            # except:
            #     print(f"\n{colorify.gray("[ SERVER ]")} Exiting...")
            #     http_server.server.shutdown()
            #     http_server.join()
            #     sys.exit(0)
            
        elif self.args.build:
            self.handle()
            
    def on_modified(self, event):
        self.handle()



def main():
    gasper = Gasper()
    gasper.run()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# coding=utf-8

"""
Copyright (c) 2024 suyambu developers (http://suyambu.net/gasper)
See the file 'LICENSE' for copying permission
"""


import json


def generate(gasper, path, exclude=None, limit=None, plugin=None, reverse=False):
    with open(path, "r") as f:
        data = json.loads(f.read())
        
    if type(data) is not list:
        data = [data]
    
    output = []
    for d in data:
        if exclude is not None:
            for ex in exclude:
                d.pop(ex)
        output.append(d)
        
    if reverse:
        output.reverse()
    
    if limit is not None:
        output = output[:limit]
    
    if plugin is not None:
        m = __import__(plugin)
        output = m.plugin(output)
    
    return output



#!/usr/bin/env python3
# coding=utf-8

"""
Copyright (c) 2024 suyambu developers (http://suyambu.net/gasper)
See the file 'LICENSE' for copying permission
"""

import re
import sys

from ..util import log
from ..util.formatter import format_space


class Args:
    def __init__(self):
        pass
    
    def __str__(self):
        return str(dir(self))


class ArgParse:
    def __init__(self, argument_space_count, usage):
        self.log = log
        self.argument_space_count = argument_space_count
        self.usage = usage
        self.commandline_arguments = []
        for arg in sys.argv[1:]:
            if arg.find("=") > -1:
                for a in arg.split("="):
                    self.commandline_arguments.append(a)
            else:
                self.commandline_arguments.append(arg)

        self.args = Args()
        self.arguments = []
    
    def __str__(self):
        return str(self.arguments)
    
    def print_help(self):
        s = "usage: {}\n\n".format(self.usage)
        for argument in self.arguments:
            other_flags = ""
            for flag in argument["other_flags"]:
                other_flags += ", {}".format(flag)
            if argument["example"] is not None:
                s += "{} : {}\n".format(format_space(argument["flag"] + other_flags + " " + argument["example"], self.argument_space_count), argument["description"])
            else:
                s += "{} : {}\n".format(format_space(argument["flag"] + other_flags, self.argument_space_count), argument["description"])
        s += """
Copyright (c) 2024 suyambu developers (http://suyambu.net/gasper)
See the file 'LICENSE' for copying permission
"""
        print(s)
        
    def set_attr(self, flag, value):
        is_set = False
        for arg in self.arguments:
            if flag in arg["other_flags"]:
                setattr(self.args, arg["flag"], value)
                is_set = True
                break
            
        if not is_set:
            setattr(self.args, flag, value)
        

    def parse(self):
        # setting up args
        for argument in self.arguments:
            flag = argument["flag"].replace("-", "")
            if argument["is_flag"]:
                self.set_attr(flag, False)
            else:
                self.set_attr(flag, None)

        # update args
        for argument in self.arguments:
            flag = argument["flag"].replace("-", "")
            for i, cmd_argument in enumerate(self.commandline_arguments):
                if (argument["flag"] == cmd_argument or cmd_argument in argument["other_flags"]) and argument["is_flag"]:
                    self.set_attr(flag, True)
                elif argument["flag"] == cmd_argument or cmd_argument in argument["other_flags"]:
                    try:
                        if argument["pattern"] is not None:
                            if re.compile(argument["pattern"]).match(self.commandline_arguments[i + 1]):
                                self.set_attr(flag, self.commandline_arguments[i + 1])
                            else:
                                print("[Error] flag {} does not match the pattern.".format(argument["flag"]))
                                sys.exit(1)
                        else:
                            self.set_attr(flag, self.commandline_arguments[i + 1])
                    except IndexError:
                        pass

        for argument in self.arguments:
            if argument["is_required"]:
                flag = argument["flag"].replace("-", "")
                if getattr(self.args, flag) is None or getattr(self.args, flag) == False:
                    self.log.error("flag {} is required".format(argument["flag"]))
                    sys.exit(1)
        return self.args

    def add_argument(self, flag, example=None, description=None, is_flag=False, pattern=None, is_required=False):
        if type(flag) is str:
            self.arguments.append({
                "flag": flag,
                "example": example,
                "description": description,
                "is_flag": is_flag,
                "pattern": pattern,
                "is_required": is_required,
                "other_flags": []
            })
        else:
            self.arguments.append({
                    "flag": flag[0],
                    "example": example,
                    "description": description,
                    "is_flag": is_flag,
                    "pattern": pattern,
                    "is_required": is_required,
                    "other_flags": flag[1:]
                })


from setuptools import setup, find_packages

setup(
    name="mullai",
    version="1.0.3",
    description="A static site generator, that will allow you to generate content from database or remote.",
    url="https://github.com/whoisjeeva/mullai",
    author="Jeeva",
    author_email="support@gumify.me",
    license="MIT",
    packages=find_packages(),
    install_requires=[
        "Jinja2==3.1.3",
        "Markdown==3.6",
        "MarkupSafe==2.1.5",
        "peewee==3.17.3",
        "PyMySQL==1.1.0",
        "python-slugify==8.0.4",
        "PyYAML==6.0.1",
        "text-unidecode==1.3",
        "watchdog==4.0.0",
        "requests==2.31.0",
        "bs4==0.0.2",
        "lxml==5.2.1"
    ],
    entry_points={
        "console_scripts": [
            "mullai = mullai.mullai:main"
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.10'
)



from jinja2 import Environment, BaseLoader, TemplateNotFound
import os
from markdown import markdown


class GasperLoader(BaseLoader):
    def __init__(self, searchpath, parent):
        self.searchpath = searchpath
        self.config = parent.config
        self.parent = parent

    def get_source(self, environment, template):
        path = os.path.join(self.searchpath, template)
        if not os.path.isfile(path):
            raise TemplateNotFound(template)
        mtime = os.path.getmtime(path)
        with open(path, 'r', encoding='utf-8') as f:
            source = f.read()
            
        matter = self.parent.global_matter
        source = self.parent.env.from_string(source).render(page=matter, content=None, site=self.config)
        if path.endswith(".md") or path.endswith(".markdown"):
            source = markdown(source, extensions=['fenced_code'])
        return source, path, lambda: mtime == os.path.getmtime(path)



#!/usr/bin/env python3
# coding=utf-8

"""
Copyright (c) 2024 suyambu developers (http://suyambu.net/gasper)
See the file 'LICENSE' for copying permission
"""

import json
import os
import platform
from shutil import get_terminal_size, rmtree
import sys
from time import gmtime, strftime
import math


def is_win():
    """Check if is a Windows system"""
    return platform.system() == "Windows"


def terminal_width():
    """Get available terminal width"""
    return get_terminal_size((80, 20)).columns


def empty_folder(path):
    """Delete a folder content recursively"""
    rmtree(path, ignore_errors=True)


def get_time():
    """Get the current time formatted by HOUR:MINUTE:SECOND"""
    return strftime("%H:%M:%S", gmtime())


def chunks(lst, n):
    """Yield successive n-sized chunks from lst."""
    number_of_list = math.ceil(len(lst)/n)
    for i in range(0, len(lst), number_of_list):
        yield lst[i:i + n]

def is_64bit():
    return (32 << bool(sys.maxsize >> 32)) == 64

def is_32bit():
    return (32 << bool(sys.maxsize >> 32)) == 32